/**
 * Copyright (C) 2007  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.bonita.deployment;

/**
 * Utility class to create a Deployment object containing the the XPDL file and optionally its
 */
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.Serializable;
import java.net.URL;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Logger;

import org.ow2.novabpm.util.Misc;
import org.xml.sax.InputSource;

/**
 * This class implements a deployment object. This object can be provided to one of the method of the ManagementAPI
 * {@link org.ow2.bonita.facade.ManagementAPI#deploy(Deployment) method} to allow the deployment of a XPDL file
 * and optionnally its depending java classes for hooks, performer assignments, mappers...
 * @author Marc Blachon, Guillaume Porcher, Charles Souillard, Miguel Valdes, Pierre Vigneras
 */
public class Deployment implements Serializable {

  private static final Logger LOG = Logger.getLogger(Deployment.class.getName());

  /**
   * XPDL definition
   */
  protected byte[] xpdl;
  /**
   * .class classes for hooks, mappers, performer assignments, multi-instantiation.
   */
  protected Set<byte[]> classes = new HashSet<byte[]>();

  /**
   * Constructs a deployment with the XPDL file and the set of classes.
   * @param xpdl The XPDL file as InputSource.
   * @param classes The set of .class classes.
   */
  public Deployment(InputSource xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    try {
      this.xpdl = Misc.getAllContentFrom(xpdl);
    } catch (IOException ioe) {
      throw new IllegalArgumentException("Couldn't open stream on input source: " + xpdl, ioe);
    }
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }

  /**
   * Constructs a deployment with the XPDL file and the set of classes.
   * @param xpdl The XPDL file as byte table.
   * @param classes The set of .class classes.
   */
  public Deployment(byte[] xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    this.xpdl = xpdl;
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }


  /**
   * Constructs a deployment with the XPDL file and the set of classes.
   * @param xpdl The XPDL file as URL.
   * @param classes The set of .class classes.
   */
  public Deployment(URL xpdl, Set<byte[]> classes) {
    Misc.checkArgsNotNull(xpdl);
    try {
      this.xpdl = Misc.getAllContentFrom(xpdl);
    } catch (IOException ioe) {
      throw new IllegalArgumentException("Couldn't open stream on url: " + xpdl, ioe);
    }
    if (classes != null) {
      this.classes.addAll(classes);
    }
  }


  private InputSource createInputSource(byte[] bytes) {
    InputStream inputStream = new ByteArrayInputStream(bytes);
    return new InputSource(inputStream);
  }

  /**
   * Returns the XPDL file as InputSource.
   * @return
   */
  public InputSource getXpdl() {
    return createInputSource(xpdl);
  }

  /**
   * Returns the .class classes as bytes table.
   * @return
   */
  public Set<byte[]> getClasses() {
    return classes;
  }

}
